import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import pystan
import pickle
import seaborn as sb

def EstMap(samples, a = 0.025, n_points = 10000):
    import numpy as np
    import scipy.stats as ss
    
    Lp, Up = np.percentile(samples, [100 * a/2, 100 * (1 - a/2)])
    coord = np.linspace(Lp, Up, n_points) 
    est_pdf = ss.gaussian_kde(samples).pdf(coord)  
    map_idx = np.argmax(est_pdf) 
    MAP_Est = coord[map_idx]   
    return MAP_Est

xlfile = pd.ExcelFile('Time_course.xlsx')
df = pd.read_excel(xlfile)
col_names = df.keys()
print('col_names:\n', col_names)
col_names = [v for v in col_names]
print('col_names:\n', col_names)
df_data = df.values
print('df_data\n', df_data)
data = np.asfarray(df_data)
print(data)
data_time = data.T[0]
data_B_ph = data.T[1]
data_Iso = data.T[2]
for t, b, i in zip(data_time, data_B_ph, data_Iso):
    print(t, b, i)

sm = pystan.StanModel(file = 'reg_exp.stan')
with open('sm.pickle', 'wb') as f:
    pickle.dump(sm, f)

stan_data = {'N':len(data_time), 'x': data_time, 'y':data_Iso}
fit_Iso = sm.sampling(data = stan_data, n_jobs = 1)
print(fit_Iso)
a_smpls_Iso = fit_Iso['a']
a_hat_Iso = EstMap(a_smpls_Iso) 
pred_Iso = 100*np.exp(a_hat_Iso * data_time)
plt.plot(data_time, pred_Iso, c = 'orange')
plt.plot(data_time, data_Iso, '^', c = 'orange', label = 'Isovaleric')

with open('sm.pickle', 'rb') as f:
    sm = pickle.load(f)
stan_data = {'N':len(data_time), 'x': data_time, 'y':data_B_ph}
fit_B_ph = sm.sampling(data = stan_data, n_jobs = 1)
print(fit_B_ph)
a_smpls_B_ph = fit_B_ph['a']
a_hat_B_ph = EstMap(a_smpls_B_ph)
pred_B_ph = 100*np.exp(a_hat_B_ph * data_time)
plt.plot(data_time, pred_B_ph, c = 'g')
plt.plot(data_time, data_B_ph, 'o', c = 'g', label = r'$\beta$-Phenyl')
plt.xlabel('Time')
plt.ylabel('Subj. intensity')
plt.legend()
plt.title('Data and Regression Curves')
plt.savefig('FigReg.png')
plt.show()

sb.kdeplot(a_smpls_Iso, c = 'orange', label = 'Isovaleric(MAP={0:.3f})'.format(a_hat_Iso))
sb.kdeplot(a_smpls_B_ph, c = 'g', label = r'$\beta$-Phenyl(MAP={0:.3f})'.format(a_hat_B_ph))
plt.legend()
plt.xlabel('a', fontsize = 14)
plt.title('Posterior Distributions')
plt.savefig('FigDistri.png')
plt.show()

print('\na_hat_Iso =', a_hat_Iso)
print('a_hat_B_ph =', a_hat_B_ph)
